@@ -170,28 +170,27 @@ class BigQueryBuilder(HasBigQueryClient):
 
 class DataFrame:
 
-    def __init__(self, query: str, alias: Optional[str], bigquery: BigQueryBuilder, **kwargs):
+    def __init__(self, query: str, alias: Optional[str], bigquery: BigQueryBuilder, deps: Optional[List['DataFrame']] = None):
         self.query = query
-        self._deps: List[Tuple[str, 'DataFrame']] = []
-        deps = kwargs.get("_deps")
-        if deps is not None:
-            self._deps = deps
+        if deps is None:
+            deps = []
+        deps = [dep for df in deps for dep in df._deps] + [(df._alias, df) for df in deps]
+        self._deps: List[Tuple[str, 'DataFrame']] = _dedup_key_value_list(deps)
         if alias is None:
             alias = bigquery._get_alias()
         else:
             bigquery._check_alias(alias, self._deps)
         self._alias = alias
-        self.bigquery = bigquery
+        self.bigquery: BigQueryBuilder = bigquery
         self._schema = None
 
     def __repr__(self):
         return f"""({self.query}) as {self._alias}"""
 
-    def _apply_query(self, query: str, deps: List[Tuple[str, 'DataFrame']] = None) -> 'DataFrame':
+    def _apply_query(self, query: str, deps: Optional[List['DataFrame']] = None) -> 'DataFrame':
         if deps is None:
-            deps = self._deps + [(self._alias, self)]
-        deps = _dedup_key_value_list(deps)
-        return DataFrame(query, None, self.bigquery, _deps=deps)
+            deps = [self]
+        return DataFrame(query, None, self.bigquery, deps=deps)
 
     def _compute_schema(self):
         df = self.limit(0)
@@ -257,9 +256,9 @@ class DataFrame:
             else:
                 raise TypeError(f"Wrong argument type: {type(columns)}")
         query = strip_margin(
-            f"""SELECT 
+            f"""SELECT
             |{cols_to_str(columns, 2)}
-            |FROM {self._alias}""")
+            |FROM {quote(self._alias)}""")
         return self._apply_query(query)
 
     def drop(self, *cols: str) -> 'DataFrame':
@@ -271,7 +270,7 @@ class DataFrame:
         query = strip_margin(
             f"""SELECT 
             |  * EXCEPT ({cols_to_str(cols)})
-            |FROM {self._alias}""")
+            |FROM {quote(self._alias)}""")
         return self._apply_query(query)
 
     def union(self, other: 'DataFrame') -> 'DataFrame':
@@ -286,8 +285,8 @@ class DataFrame:
         :param other:
         :return: a new :class:`DataFrame`
         """
-        query = f"""SELECT * FROM {self._alias} UNION ALL SELECT * FROM {other._alias}"""
-        return self._apply_query(query, deps=self._deps + other._deps + [(self._alias, self), (other._alias, other)])
+        query = f"""SELECT * FROM {quote(self._alias)} UNION ALL SELECT * FROM {quote(other._alias)}"""
+        return self._apply_query(query, deps=[self, other])
 
     unionAll = union
 
@@ -365,19 +364,19 @@ class DataFrame:
         |  {cols_to_str(self_only_cols, 2)}{optional_comma(self_only_cols)}
         |  {cols_to_str(common_cols, 2)}{optional_comma(common_cols)}
         |  {cols_to_str([f"NULL as {col}" for col in other_only_cols], 2)}
-        |FROM {self._alias} 
+        |FROM {quote(self._alias)} 
         |UNION ALL 
         |SELECT 
         |  {cols_to_str([f"NULL as {col}" for col in self_only_cols], 2)}{optional_comma(self_only_cols)}
         |  {cols_to_str(common_cols, 2)}{optional_comma(common_cols)}
         |  {cols_to_str(other_only_cols, 2)}
-        |FROM {other._alias}
+        |FROM {quote(other._alias)}
         |""")
-        return self._apply_query(query, deps=self._deps + other._deps + [(self._alias, self), (other._alias, other)])
+        return self._apply_query(query, deps=[self, other])
 
     def limit(self, num: int) -> 'DataFrame':
         """Returns a new :class:`DataFrame` with a result count limited to the specified number of rows."""
-        query = f"""SELECT * FROM {self._alias} LIMIT {num}"""
+        query = f"""SELECT * FROM {quote(self._alias)} LIMIT {num}"""
         return self._apply_query(query)
 
     def distinct(self) -> 'DataFrame':
@@ -387,7 +386,7 @@ class DataFrame:
         -----------------------------
         In BigQuery, the DISTINCT statement does not work on complex types like STRUCT and ARRAY.
         """
-        query = f"""SELECT DISTINCT * FROM {self._alias}"""
+        query = f"""SELECT DISTINCT * FROM {quote(self._alias)}"""
         return self._apply_query(query)
 
     def sort(self, *cols: str):
@@ -398,7 +397,7 @@ class DataFrame:
         """
         query = strip_margin(f"""
         |SELECT * 
-        |FROM {self._alias} 
+        |FROM {quote(self._alias)} 
         |ORDER BY {cols_to_str(cols)}""")
         return self._apply_query(query)
 
@@ -408,7 +407,7 @@ class DataFrame:
         """Filters rows using the given condition."""
         query = strip_margin(f"""
         |SELECT * 
-        |FROM {self._alias} 
+        |FROM {quote(self._alias)} 
         |WHERE {expr}""")
         return self._apply_query(query)
 
@@ -434,14 +433,14 @@ class DataFrame:
         :return: a new :class:`DataFrame`
         """
         if replace:
-            query = f"SELECT * REPLACE ({col_expr} AS {col_name}) FROM {self._alias}"
+            query = f"SELECT * REPLACE ({col_expr} AS {col_name}) FROM {quote(self._alias)}"
         else:
-            query = f"SELECT *, {col_expr} AS {col_name} FROM {self._alias}"
+            query = f"SELECT *, {col_expr} AS {col_name} FROM {quote(self._alias)}"
         return self._apply_query(query)
 
     def count(self) -> int:
         """Returns the number of rows in this :class:`DataFrame`."""
-        query = f"SELECT COUNT(1) FROM {self._alias}"
+        query = f"SELECT COUNT(1) FROM {quote(self._alias)}"
         return self._apply_query(query).collect()[0][0]
 
     def collect_iterator(self) -> RowIterator:
@@ -507,6 +506,29 @@ class DataFrame:
         """
         print(self.treeString())
 
+    def print_query(self) -> None:
+        """Prints out the SQL query generated to materialize this :class:`DataFrame`.
+
+        Examples:
+
+        >>> bq = BigQueryBuilder(get_bq_client())
+        >>> from bigquery_frame import functions as f
+        >>> df = bq.sql('''SELECT 1 as a''').select('a', f.col('a')+f.lit(1).alias('b')).withColumn('c', f.expr('a + b'))
+        >>> df.print_query()
+        WITH `_default_alias_1` AS (
+          SELECT 1 as a
+        )
+        , `_default_alias_2` AS (
+          SELECT
+            a,
+            a + 1
+          FROM `_default_alias_1`
+        )
+        SELECT *, a + b AS c FROM `_default_alias_2`
+
+        """
+        print(self.compile())
+
     @property
     def columns(self) -> List[str]:
         """Returns all column names as a list."""
